<?php
/**
 * @file
 * All forms for opac register module.
 */


/**
 * Form constructor for opac registration form.
 *
 * Allows a user to type his email address.
 *
 * @see opac_register_registration_form_validate()
 * @ingroup forms
 */
function opac_register_registration_form($form, &$form_state) {

  if (user_is_anonymous()) {

    $form['person_type'] = array(
      '#type' => 'radios',
      '#default_value' => 'natural',
      '#options' => array(
        'natural' => t('I am a natural person'),
        'collectivity' => t('I am a collectivity'),
      ),
      '#required' => TRUE,
    );

    $form['drupal_register'] = array(
      '#type' => 'fieldset',
      '#title' => t('Personal informations'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#attributes' => array(
        'id' => 'opac-register-drupal-register',
      ),
    );
    $form['drupal_register']['title'] = array(
      '#type' => 'select',
      '#title' => t('Title'),
      '#default_value' => 'miss',
      '#options' => array(
        'mrs' => t('Mrs'),
        'mr' => t('Mr.'),
      ),
      '#states' => array(
        'visible' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
        'required' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
      ),
    );
    $form['drupal_register']['firstname'] = array(
      '#type' => 'textfield',
      '#title' => t('Firstname'),
      '#size' => 30,
      '#states' => array(
        'visible' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
        'required' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
      ),
    );
    $form['drupal_register']['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#size' => 30,
      '#states' => array(
        'visible' => array(
          ':input[name="person_type"]' => array('value' => 'collectivity'),
        ),
        'required' => array(
          ':input[name="person_type"]' => array('value' => 'collectivity'),
        ),
      ),
    );
    $form['drupal_register']['surname'] = array(
      '#type' => 'textfield',
      '#title' => t('Surname'),
      '#size' => 30,
      '#states' => array(
        'visible' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
        'required' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
      ),
    );
    $form['drupal_register']['login'] = array(
      '#type' => 'textfield',
      '#title' => t('login'),
      '#size' => 60,
      '#required' => TRUE,
    );
    $form['drupal_register']['password'] = array(
      '#type' => 'password',
      '#title' => t('password'),
      '#size' => 60,
      '#required' => TRUE,
    );
    $form['drupal_register']['password_confirm'] = array(
      '#title' => t('Password confirmation'),
      '#description' => t('Please enter the same password again.'),
      '#type' => t('password'),
      '#size' => 60,
    );
    $form['drupal_register']['email'] = array(
      '#type' => 'textfield',
      '#title' => t('Email'),
      '#size' => 60,
      '#required' => TRUE,
      '#rules' => array(
        array('rule' => 'email', 'error' => 'Email is not valid.'),
      ),
    );
    $form['drupal_register']['dateofbirth'] = array(
      '#type' => 'date',
      '#title' => t('Date of birth'),
      '#states' => array(
        'visible' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
        'required' => array(
          ':input[name="person_type"]' => array('value' => 'natural'),
        ),
      ),
    );
    $form['drupal_register']['conditions'] = array(
      '#type' => 'checkbox',
      '#title' => t('I agree with the ') . l(t('terms of service.'), 'opac-register/drupal-terms'),
      '#required' => TRUE,
    );

    $form['want_libray_registration'] = array(
      '#type' => 'checkbox',
      '#title' => t('I want to pre-register to the library.'),
    );
    $form['library_register'] = array(
      '#type' => 'fieldset',
      '#title' => t('Personal informations for library registration'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
      '#states' => array(
        'visible' => array(
          ':input[name="want_libray_registration"]' => array('checked' => TRUE),
        ),
      ),
    );

  }
  else {
    $form['want_libray_registration'] = array(
      '#type' => 'hidden',
      '#value' => '1',
    );
    $form['library_register'] = array(
      '#type' => 'fieldset',
      '#title' => t('Personal informations for library registration'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    );

    $form['library_register']['password'] = array(
      '#type' => 'password',
      '#title' => t('password'),
      '#size' => 60,
      '#required' => TRUE,
      '#description' => "You have to enter this site's password.",
    );
  }

  // TODO: Do not display list if there is only one server.
  $servers = opac_register_ils_servers();
  $form['library_register']['server'] = array(
    '#type' => 'select',
    '#title' => t('Library'),
    '#options' => $servers,
  );

  $form['library_register']['branch'] = array(
    '#type' => 'select',
    '#title' => t('Branch'),
    '#options' => opac_register_user_branches(),
  );
  $form['library_register']['zip_code'] = array(
    '#type' => 'textfield',
    '#title' => t('Zip code'),
    '#size' => 10,
  );
  $form['library_register']['address'] = array(
    '#type' => 'textfield',
    '#title' => t('Address'),
    '#size' => 50,
  );
  $form['library_register']['city'] = array(
    '#type' => 'textfield',
    '#title' => t('City'),
    '#size' => 30,
  );
  $form['library_register']['phone'] = array(
    '#type' => 'textfield',
    '#title' => t('Phone'),
    '#size' => 30,
  );
  $form['library_register']['terms_use'] = array(
    '#type' => 'checkbox',
    '#title' => t('I accept the ') . l(t('library terms of use.'), 'opac-register/library-terms'),
  );

  $form['info_accurate'] = array(
    '#type' => 'checkbox',
    '#title' => t('I certify that this information is accurate.'),
    '#type' => 'submit',
    '#value' => t('Register'),
  );
  return $form;
}

/**
 * Validation handler for opac_register_registration_form().
 */
function opac_register_registration_form_validate($form, &$form_state) {
  global $user;
  $values = $form_state['values'];
  module_load_include('inc', 'opac', 'includes/opac.harvest');

  $connector = opac_get_instance(opac_server_load($values['server']));

  if (user_is_anonymous()) {

    $password = $form_state['values']['password'];
    $password_confirm = $form_state['values']['password_confirm'];

    if (strlen($password) < 6) {
      form_set_error('password', t('Your password must be at least 6 characters.'));
    }

    if ($password != $password_confirm) {
      form_set_error('password_confirm', t("Passwords don't match"));
    }

    if ($values['person_type'] == 'natural') {
      if (!$values['title']) {
        form_set_error('title', t('Title field is required'));
      }
      if (!$values['firstname']) {
        form_set_error('firstname', t('Firstname field is required'));
      }
      if (!$values['surname']) {
        form_set_error('surname', t('Surname field is required'));
      }
    }
    else {
      if (!$values['name']) {
        form_set_error('name', t('Name field is required'));
      }
    }

    if (form_get_errors()) {
      return;
    }
  }
  $email = isset($values['email']) ? $values['email'] : $user->mail;
  $login = isset($values['login']) ? $values['login'] : $user->name;

  // Check if login exists in ILS.
  $response = $connector->loginExists($login);

  if ($response) {
    form_set_error('opac_register_registration_form', t('This login already exists in the ILS. Please choose another one or contact the library.'));
  }

  // Check if user exists in Drupal.
  if (user_is_anonymous()) {
    $exists_drupal = FALSE;
    if (opac_register_user_exists(array('name' => $login))) {
      $exists_drupal = TRUE;
    }
    if (opac_register_user_exists(array('mail' => $email))) {
      $exists_drupal = TRUE;
    }

    if ($exists_drupal) {
      form_set_error('opac_register_registration_form', t('This login or email already exists in Drupal. Please choose another one.'));
    }
  }
}

/**
 * Submit handler for opac_register_registration_form().
 */
function opac_register_registration_form_submit($form, &$form_state) {
  global $user;
  $user_data = user_load($user->uid);
  $values = $form_state['values'];
  $opac_server_identifier = opac_server_load($values['server']);
  $connector = opac_get_instance($opac_server_identifier);
  $want_libray_registration = FALSE;

  $email = isset($values['email']) ? $values['email'] : $user->mail;
  $login = isset($values['login']) ? $values['login'] : $user->name;
  $firstname = isset($values['firstname']) ? $values['firstname'] : $user_data->field_account_firstname['und'][0]['value'];
  $surname = isset($values['surname']) ? $values['surname'] : $user_data->field_account_surname['und'][0]['value'];
  if ($firstname == '' || $surname == '') {
    $firstname = $surname = $values['name'];
  }
  $title = isset($values['title']) ? $values['title'] : $user_data->field_account_title['und'][0]['value'];
  $dateofbirth = isset($values['dateofbirth']) ?
    $values['dateofbirth']['year'] . '-' . sprintf("%02d", $values['dateofbirth']['month']) . '-' . sprintf("%02d", $values['dateofbirth']['day']) : 
    $user_data->field_account_dateofbirth['und'][0]['value'];


  if (user_is_anonymous()) {
    $drupal_user = array(
      'name' => $login,
      'mail' => $email,
      'pass' => $values['password'],
      'status' => 1,
      'roles' => array(
        DRUPAL_AUTHENTICATED_RID => 'authenticated user',
      ),
      'field_account_firstname' => array(
        'und' => array(
          0 => array(
            'value' => $firstname,
          ),
        ),
      ),
      'field_account_surname' => array(
        'und' => array(
          0 => array(
            'value' => $surname,
          ),
        ),
      ),
      'field_account_title' => array(
        'und' => array(
          0 => array(
            'value' => $title,
          ),
        ),
      ),
      'field_account_dateofbirth' => array(
        'und' => array(
          0 => array(
            'value' => $dateofbirth,
          ),
        ),
      ),
    );

    // Ugly (but needed) fix for the CAS module.
    if (module_exists('cas')) {
      $drupal_user['cas_name'] = $login;
    }

    // Let's save the user.
    $account = user_save('', $drupal_user);

    if ($account) {
      drupal_set_message(t("Your drupal account has been successfully created. You can now login with @login as username.", array('@login' => $login)));
      $params['account'] = $account;
      $send = drupal_mail(
        'opac_register',
        'confirm_drupal_register',
        $account->mail,
        language_default(),
        $params
      );

    }
    else {
      form_set_error(t("Your drupal account could not be created."));
    }
  }

  if ($values['want_libray_registration']) {
    $ils_user = array(
      'userid' => $login,
      'password' => $values['password'],
      'firstname' => $firstname,
      'surname' => $surname,
      'branchcode' => $values['branch'],
    );

    if (isset($sex)) {
      $ils_user['sex'] = $sex;
    }
    $ils_user['title'] = $title;
    $ils_user['dateofbirth'] = $dateofbirth;
    $ils_user['email'] = $email;
    if ($values['zip_code']) {
      $ils_user['zipcode'] = $values['zip_code'];
    }
    if ($values['address']) {
      $ils_user['address'] = $values['address'];
    }
    if ($values['city']) {
      $ils_user['city'] = $values['city'];
    }
    if ($values['phone']) {
      $ils_user['phone'] = $values['phone'];
    }

    // We create the user in the ILS.
    $result = $connector->createUser($ils_user);

    /* But we also change (or set) the opac server identifier in the
    drupal account. */
    $drupal_user_edit = array(
      variable_get('opac_users_server_field', '') => array(
        'und' => array(
          0 => array(
            'value' => $values['server'],
          ),
        ),
      ),
    );
    /* If the account was defined (drupal and ils registration at the same
    time), we use it, else, the ils registration took place after the drupal
    registration, so we use the logged-in user informations. */
    if (!isset($account)) {
      $account = $user;
    }
    user_save($account, $drupal_user_edit);

    if (!$result) {
      form_set_error('opac_register_registration_form', t('Registration process failed.'));
    }
    else {
      drupal_set_message(t('Your ILS registration has been successfully processed.'));
      $params['account'] = $account;
      $params['ilsname'] = $values['server'];
      $send = drupal_mail(
        'opac_register',
        'confirm_ils_register',
        $account->mail,
        language_default(),
        $params
      );
    }
  }
}
