<?php

/**
 * @file
 * Theme hooks.
 */

/**
 * Implements hook_html_head_alter().
 *
 * This will overwrite the default meta character type tag with HTML5 version.
 */
function wheke_responsive_theme_html_head_alter(&$head_elements) {
  $head_elements['system_meta_content_type']['#attributes'] = array(
    'charset' => 'utf-8',
  );
}

/**
 * Insert themed breadcrumb page navigation at top of the node content.
 */
function wheke_responsive_theme_breadcrumb($variables) {
  $breadcrumb = $variables['breadcrumb'];
  if (!empty($breadcrumb)) {
    // Use CSS to hide titile .element-invisible.
    $output = '<h2 class="element-invisible">' . t('You are here') . '</h2>';
    // Comment below line to hide current page to breadcrumb.
    $breadcrumb[] = drupal_get_title();
    $output .= '<nav class="breadcrumb">' . implode(' » ', $breadcrumb) . '</nav>';
    return $output;
  }
}

/**
 * Override or insert variables into the page template.
 */
function wheke_responsive_theme_preprocess_page(&$vars) {
  if (isset($vars['main_menu'])) {
    $main_menu_tree = array();
    if (module_exists('i18n')) {
      $main_menu_tree = i18n_menu_translated_tree(variable_get('menu_main_links_source', 'main-menu'));
    }
    else {
      $main_menu_tree = menu_tree(variable_get('menu_main_links_source', 'main-menu'));
    }
    $vars['main_menu'] = $main_menu_tree;
  }
  else {
    $vars['main_menu'] = FALSE;
  }

  if (isset($vars['secondary_menu'])) {
    $vars['secondary_menu'] = theme('links__system_secondary_menu', array(
      'links' => $vars['secondary_menu'],
      'attributes' => array(
        'class' => array('links', 'secondary-menu', 'clearfix'),
      ),
      'heading' => array(
        'text' => t('Secondary menu'),
        'level' => 'h2',
        'class' => array('element-invisible'),
      ),
    ));
  }
  else {
    $vars['secondary_menu'] = FALSE;
  }
}

/**
 * Duplicate of theme_menu_local_tasks() but adds clearfix to tabs.
 */
function wheke_responsive_theme_menu_local_tasks(&$variables) {
  $output = '';

  if (!empty($variables['primary'])) {
    $variables['primary']['#prefix'] = '<h2 class="element-invisible">' . t('Primary tabs') . '</h2>';
    $variables['primary']['#prefix'] .= '<ul class="tabs primary clearfix">';
    $variables['primary']['#suffix'] = '</ul>';
    $output .= drupal_render($variables['primary']);
  }
  if (!empty($variables['secondary'])) {
    $variables['secondary']['#prefix'] = '<h2 class="element-invisible">' . t('Secondary tabs') . '</h2>';
    $variables['secondary']['#prefix'] .= '<ul class="tabs secondary clearfix">';
    $variables['secondary']['#suffix'] = '</ul>';
    $output .= drupal_render($variables['secondary']);
  }
  return $output;
}

/**
 * Override or insert variables into the node template.
 */
function wheke_responsive_theme_preprocess_node(&$variables) {
  $node = $variables['node'];
  if ($variables['view_mode'] == 'full' && node_is_page($variables['node'])) {
    $variables['classes_array'][] = 'node-full';
  }
}

/**
 * Implements hook_page_alter().
 */
function wheke_responsive_theme_page_alter($page) {
  /*
   * <meta name="viewport"
   *       content="width=device-width, initial-scale=1, maximum-scale=1" />
   */
  $viewport = array(
    '#type' => 'html_tag',
    '#tag' => 'meta',
    '#attributes' => array(
      'name' => 'viewport',
      'content' => 'width=device-width',
    ),
  );
  drupal_add_html_head($viewport, 'viewport');
}

/**
 * Implements hook_views_post_render().
 *
 * Add a CSS when the 'calendrier' view is rendered.
 */
function wheke_responsive_theme_views_post_render(&$view) {
  if ($view->name == 'calendrier' && $view->current_display == 'block_1') {
    $path = drupal_get_path('theme', 'wheke_responsive_theme');
    drupal_add_css($path . '/css/wrt_views-view-calendrier-block_1.css');
  }
}
